(* a simple NCurses UI Library *)
open Curses

type point = int * int (* x and y *)

type buffer = string list

exception ScreenNotOpen
exception NoRootWidget

type 'a member = {
    set : 'a -> unit;
    get : unit -> 'a
}

type widget = {
    minSize: point member;
    paint: point -> buffer;
}

let member_init (v: 'a): 'a member =
    let pointer = ref (v) in
    {
        set = (fun n -> pointer := v);
        get = (fun () -> !pointer);
    }

let label str = {
        minSize = member_init (0,0);
        paint = (fun (w,h) -> [ str ]);
    }

let root_widget : widget option ref = ref (None)

let get_root_widget () : widget =
    match (!root_widget) with
    | None -> raise NoRootWidget
    | Some x -> x

let stdscr : window option ref = ref (None)

let screen_open  () =
    stdscr := Some (initscr ())

let get_stdscr () =
    match (!stdscr) with
    | None -> raise ScreenNotOpen
    | Some x -> x

let screen_close () =
    stdscr := None;
    endwin ()

let paint_widget (wid:widget) ((x,y):point) (size: point) =
    let stdscr = get_stdscr () in
    let lines = wid.paint size in
    let paint i str =
        mvwaddstr stdscr (y+i) x str
    in
    List.iteri paint lines

let update () =
    let h, w = getmaxyx (get_stdscr ()) in
    paint_widget (get_root_widget()) (0,0) (w,h);
    refresh()

let set_root_widget widget =
    root_widget := Some widget;
    update()

