

type annotation = string * int * int (* file and line number and column*)

exception ParseError of annotation * string
exception OWLParseError of string

type 'a annotated = 'a * annotation

type 'a tree = (* tree carrying values only in leafes *)
    | Leaf of 'a
    | Node of (('a tree) list)

type 'a intree = (* tree carrying values in leafes and inner nodes *)
    | INode of 'a * ('a intree list)

val intree_list_of_tree : 'a tree -> 'a intree list
val tree_of_intree : 'a intree -> 'a tree
val intree_flatten : 'a intree -> 'a list
val list_of_stream : 'a Stream.t -> 'a list

val string_of_annotation : annotation -> string
val string_of_annotated : ('a -> string) -> 'a annotated -> string

val filter_comment : string -> string
val filter_whitespace : string annotated list -> string annotated list

val tokens_of_string_stream : string Stream.t-> string annotated list
val tree_of_tokens : string annotated list -> string annotated tree
val string_of_tree : ('a -> string) -> 'a tree -> string
val string_of_intree : ('a -> string) -> 'a intree -> string

val tree_of_string : string -> string annotated tree


val parse : string -> OWL.prefixes * OWL.ontology list
val parse_intree_list : string annotated intree list -> OWL.prefixes * (OWL.ontology list)
